@@ -32,19 +32,16 @@ extern "C"
 #include "qgsvirtuallayerprovider.h"
 #include "qgsvirtuallayersqlitemodule.h"
 #include "qgsvirtuallayerqueryparser.h"
+#include "qgsapplication.h"
 
 #include <QUrl>
+#include <QUrlQuery>
 
 const QString QgsVirtualLayerProvider::VIRTUAL_LAYER_KEY = QStringLiteral( "virtual" );
 const QString QgsVirtualLayerProvider::VIRTUAL_LAYER_DESCRIPTION = QStringLiteral( "Virtual layer data provider" );
 const QString QgsVirtualLayerProvider::VIRTUAL_LAYER_QUERY_VIEW = QStringLiteral( "_query" );
 
-static QString quotedColumn( QString name )
-{
-  return "\"" + name.replace( QLatin1String( "\"" ), QLatin1String( "\"\"" ) ) + "\"";
-}
-
-#define PROVIDER_ERROR( msg ) do { mError = QgsError( msg, QgsVirtualLayerProvider::VIRTUAL_LAYER_KEY ); QgsDebugMsg( msg ); } while(0)
+#define PROVIDER_ERROR( msg ) do { mError = QgsError( msg, QgsVirtualLayerProvider::VIRTUAL_LAYER_KEY ); QgsDebugError( msg ); } while(0)
 
 
 QgsVirtualLayerProvider::QgsVirtualLayerProvider( QString const &uri,
@@ -53,6 +50,7 @@ QgsVirtualLayerProvider::QgsVirtualLayerProvider( QString const &uri,
   : QgsVectorDataProvider( uri, options, flags )
 {
   mError.clear();
+  mExtent.setNull(); // ideally not needed
 
   const QUrl url = QUrl::fromEncoded( uri.toUtf8() );
   if ( !url.isValid() )
@@ -120,7 +118,7 @@ bool QgsVirtualLayerProvider::loadSourceLayers()
         PROVIDER_ERROR( QString( "Cannot find layer %1" ).arg( layer.reference() ) );
         return false;
       }
-      if ( l->type() != QgsMapLayerType::VectorLayer )
+      if ( l->type() != Qgis::LayerType::Vector )
       {
         PROVIDER_ERROR( QString( "Layer %1 is not a vector layer" ).arg( layer.reference() ) );
         return false;
@@ -249,7 +247,7 @@ bool QgsVirtualLayerProvider::createIt()
       const auto constMapLayers = QgsProject::instance()->mapLayers();
       for ( const QgsMapLayer *l : constMapLayers )
       {
-        if ( l->type() != QgsMapLayerType::VectorLayer )
+        if ( l->type() != Qgis::LayerType::Vector )
           continue;
 
         const QgsVectorLayer *vl = static_cast<const QgsVectorLayer *>( l );
@@ -292,7 +290,7 @@ bool QgsVirtualLayerProvider::createIt()
   resetSqlite();
   initVirtualLayerMetadata( mSqlite.get() );
 
-  const bool noGeometry = mDefinition.geometryWkbType() == QgsWkbTypes::NoGeometry;
+  const bool noGeometry = mDefinition.geometryWkbType() == Qgis::WkbType::NoGeometry;
 
   // load source layers (and populate mLayers)
   if ( !loadSourceLayers() )
@@ -532,9 +530,9 @@ bool QgsVirtualLayerProvider::setSubsetString( const QString &subset, bool updat
 }
 
 
-QgsWkbTypes::Type QgsVirtualLayerProvider::wkbType() const
+Qgis::WkbType QgsVirtualLayerProvider::wkbType() const
 {
-  return static_cast<QgsWkbTypes::Type>( mDefinition.geometryWkbType() );
+  return static_cast<Qgis::WkbType>( mDefinition.geometryWkbType() );
 }
 
 long long QgsVirtualLayerProvider::featureCount() const
@@ -557,12 +555,25 @@ QgsRectangle QgsVirtualLayerProvider::extent() const
 
 void QgsVirtualLayerProvider::updateStatistics() const
 {
-  const bool hasGeometry = mDefinition.geometryWkbType() != QgsWkbTypes::NoGeometry;
-  const QString subset = mSubset.isEmpty() ? QString() : " WHERE " + mSubset;
-  const QString sql = QStringLiteral( "SELECT Count(*)%1 FROM %2%3" )
-                      .arg( hasGeometry ? QStringLiteral( ",Min(MbrMinX(%1)),Min(MbrMinY(%1)),Max(MbrMaxX(%1)),Max(MbrMaxY(%1))" ).arg( quotedColumn( mDefinition.geometryField() ) ) : QString(),
-                            mTableName,
-                            subset );
+  const bool hasGeometry = mDefinition.geometryWkbType() != Qgis::WkbType::NoGeometry;
+
+  QString sql = QStringLiteral( "SELECT Count(1)" );
+
+  if ( hasGeometry )
+  {
+    sql += QStringLiteral(
+             ", Min(MbrMinX(%1)), Min(MbrMinY(%1)), Max(MbrMaxX(%1)), Max(MbrMaxY(%1))"
+           ).arg( QgsSqliteUtils::quotedIdentifier( mDefinition.geometryField() ) );
+  }
+
+  sql += QStringLiteral( " FROM %1" ) .arg( mTableName );
+
+  if ( !mSubset.isEmpty() )
+  {
+    sql += " WHERE ( " + mSubset + ')';
+  }
+
+  mExtent.setNull();
 
   try
   {
@@ -570,7 +581,7 @@ void QgsVirtualLayerProvider::updateStatistics() const
     if ( q.step() == SQLITE_ROW )
     {
       mFeatureCount = q.columnInt64( 0 );
-      if ( hasGeometry )
+      if ( mFeatureCount && hasGeometry )
       {
         double x1, y1, x2, y2;
         x1 = q.columnDouble( 1 );
@@ -593,6 +604,7 @@ void QgsVirtualLayerProvider::updateStatistics() const
 void QgsVirtualLayerProvider::invalidateStatistics()
 {
   mCachedStatistics = false;
+  emit dataChanged();
 }
 
 QgsFields QgsVirtualLayerProvider::fields() const
@@ -670,11 +682,114 @@ QgsVirtualLayerProvider *QgsVirtualLayerProviderMetadata::createProvider(
   return new QgsVirtualLayerProvider( uri, options, flags );
 }
 
+QString QgsVirtualLayerProviderMetadata::absoluteToRelativeUri( const QString &uri, const QgsReadWriteContext &context ) const
+{
+  QUrl urlSource = QUrl::fromEncoded( uri.toLatin1() );
+  QStringList theURIParts;
+
+  QUrlQuery query = QUrlQuery( urlSource.query() );
+  QList<QPair<QString, QString> > queryItems = query.queryItems();
+
+  for ( int i = 0; i < queryItems.size(); i++ )
+  {
+    QString key = queryItems.at( i ).first;
+    QString value = queryItems.at( i ).second;
+    if ( key == QLatin1String( "layer" ) )
+    {
+      // syntax: provider:url_encoded_source_URI(:name(:encoding)?)?
+      theURIParts = value.split( ':' );
+      theURIParts[1] = QUrl::fromPercentEncoding( theURIParts[1].toUtf8() );
+
+      if ( theURIParts[0] == QLatin1String( "delimitedtext" ) )
+      {
+        QUrl urlSource = QUrl( theURIParts[1] );
+        QUrl urlDest = QUrl::fromLocalFile( context.pathResolver().writePath( urlSource.toLocalFile() ) );
+        urlDest.setQuery( urlSource.query() );
+        theURIParts[1] = QUrl::toPercentEncoding( urlDest.toString(), QByteArray( "" ), QByteArray( ":" ) );
+      }
+      else
+      {
+        theURIParts[1] = context.pathResolver().writePath( theURIParts[1] );
+        theURIParts[1] = QUrl::toPercentEncoding( theURIParts[1] );
+      }
+
+      queryItems[i].second =  theURIParts.join( QLatin1Char( ':' ) ) ;
+    }
+  }
+
+  query.setQueryItems( queryItems );
+
+  QUrl urlDest = QUrl( urlSource );
+  urlDest.setQuery( query.query() );
+  return QString::fromLatin1( urlDest.toEncoded() );
+}
+
+QString QgsVirtualLayerProviderMetadata::relativeToAbsoluteUri( const QString &uri, const QgsReadWriteContext &context ) const
+{
+  QUrl urlSource = QUrl::fromEncoded( uri.toLatin1() );
+  QStringList theURIParts;
+
+  QUrlQuery query = QUrlQuery( urlSource.query() );
+  QList<QPair<QString, QString> > queryItems = query.queryItems();
+
+  for ( int i = 0; i < queryItems.size(); i++ )
+  {
+    QString key = queryItems.at( i ).first;
+    QString value = queryItems.at( i ).second;
+    if ( key == QLatin1String( "layer" ) )
+    {
+      // syntax: provider:url_encoded_source_URI(:name(:encoding)?)?
+      theURIParts = value.split( ':' );
+      theURIParts[1] = QUrl::fromPercentEncoding( theURIParts[1].toUtf8() );
+
+      if ( theURIParts[0] == QLatin1String( "delimitedtext" ) )
+      {
+        QUrl urlSource = QUrl( theURIParts[1] );
+
+        if ( !theURIParts[1].startsWith( QLatin1String( "file:" ) ) )
+        {
+          QUrl file = QUrl::fromLocalFile( theURIParts[1].left( theURIParts[1].indexOf( '?' ) ) );
+          urlSource.setScheme( QStringLiteral( "file" ) );
+          urlSource.setPath( file.path() );
+        }
+
+        QUrl urlDest = QUrl::fromLocalFile( context.pathResolver().readPath( urlSource.toLocalFile() ) );
+        urlDest.setQuery( urlSource.query() );
+
+        theURIParts[1] = urlDest.toString();
+      }
+      else
+      {
+        theURIParts[1] = context.pathResolver().readPath( theURIParts[1] );
+      }
+
+      theURIParts[1] = QUrl::toPercentEncoding( theURIParts[1] );
+      queryItems[i].second =  theURIParts.join( QLatin1Char( ':' ) ) ;
+    }
+  }
+
+  query.setQueryItems( queryItems );
+
+  QUrl urlDest = QUrl( urlSource );
+  urlDest.setQuery( query.query() );
+  return QString::fromLatin1( urlDest.toEncoded() );
+}
+
+QList<Qgis::LayerType> QgsVirtualLayerProviderMetadata::supportedLayerTypes() const
+{
+  return { Qgis::LayerType::Vector };
+}
+
 QgsVirtualLayerProviderMetadata::QgsVirtualLayerProviderMetadata():
   QgsProviderMetadata( QgsVirtualLayerProvider::VIRTUAL_LAYER_KEY, QgsVirtualLayerProvider::VIRTUAL_LAYER_DESCRIPTION )
 {
 }
 
+QIcon QgsVirtualLayerProviderMetadata::icon() const
+{
+  return QgsApplication::getThemeIcon( QStringLiteral( "mIconVirtualLayer.svg" ) );
+}
+
 
 #ifndef HAVE_STATIC_PROVIDERS
 QGISEXTERN QgsProviderMetadata *providerMetadataFactory()
