@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """QGIS Unit tests for QgsLegendPatchShape.
 
 .. note:: This program is free software; you can redistribute it and/or modify
@@ -11,37 +10,31 @@ __date__ = '05/04/2020'
 __copyright__ = 'Copyright 2020, The QGIS Project'
 
 import qgis  # NOQA
+from qgis.PyQt.QtCore import QDir, QSize, QSizeF
+from qgis.PyQt.QtGui import QColor, QImage, QPainter
+from qgis.PyQt.QtXml import QDomDocument
+from qgis.core import (
+    QgsFillSymbol,
+    QgsGeometry,
+    QgsLegendPatchShape,
+    QgsLineSymbol,
+    QgsMarkerSymbol,
+    QgsReadWriteContext,
+    QgsRenderChecker,
+    QgsRenderContext,
+    QgsStyle,
+    QgsSymbol,
+)
+import unittest
+from qgis.testing import start_app, QgisTestCase
 
-from qgis.PyQt.QtCore import (QSize,
-                              QSizeF,
-                              QPointF,
-                              QDir)
-from qgis.PyQt.QtGui import (QPolygonF,
-                             QImage,
-                             QPainter,
-                             QColor)
-from qgis.core import (QgsLegendPatchShape,
-                       QgsGeometry,
-                       QgsSymbol,
-                       QgsFillSymbol,
-                       QgsLineSymbol,
-                       QgsMarkerSymbol,
-                       QgsRenderChecker,
-                       QgsReadWriteContext,
-                       QgsRenderContext,
-                       QgsStyle
-                       )
-from qgis.PyQt.QtXml import QDomDocument, QDomElement
-
-from qgis.testing import start_app, unittest
 from utilities import unitTestDataPath
 
-
 start_app()
 TEST_DATA_DIR = unitTestDataPath()
 
 
-class TestQgsLegendPatchShape(unittest.TestCase):
+class TestQgsLegendPatchShape(QgisTestCase):
 
     def setUp(self):
         # Create some simple symbols
@@ -51,7 +44,7 @@ class TestQgsLegendPatchShape(unittest.TestCase):
         self.report = "<h1>Python QgsLegendPatchShape Tests</h1>\n"
 
     def tearDown(self):
-        report_file_path = "%s/qgistest.html" % QDir.tempPath()
+        report_file_path = f"{QDir.tempPath()}/qgistest.html"
         with open(report_file_path, 'a') as report_file:
             report_file.write(self.report)
 
@@ -281,7 +274,7 @@ class TestQgsLegendPatchShape(unittest.TestCase):
         return image
 
     def imageCheck(self, name, reference_image, image):
-        self.report += "<h2>Render {}</h2>\n".format(name)
+        self.report += f"<h2>Render {name}</h2>\n"
         temp_dir = QDir.tempPath() + '/'
         file_name = temp_dir + 'patch_' + name + ".png"
         image.save(file_name, "PNG")
@@ -292,7 +285,7 @@ class TestQgsLegendPatchShape(unittest.TestCase):
         checker.setColorTolerance(2)
         result = checker.compareImages(name, 20)
         self.report += checker.report()
-        print((self.report))
+        print(self.report)
         return result
 
 
